GuildWindowFix = {}

function FormatLastLoginTime(_timeInSeconds, _hour, _month, _day, _year)
	local tempRoundedNumber = 0

	if _timeInSeconds < 86400 then -- 60*60*24 = 86400	-- Last Login/Logout was less than 24 hours ago
		tempRoundedNumber = math.floor(_timeInSeconds / 86400)
		return GetFormatStringFromTable( "GuildStrings", StringTables.Guild.DATE_FORMAT_ROSTER_LOGIN_X_HOURS_AGO, {tempRoundedNumber} ) 
	end	

	if _timeInSeconds < 259200 then -- 86400*3= 259200	-- Last Login/logout was less than 2 days ago
		tempRoundedNumber = math.floor(_timeInSeconds / 259200)
		return GetFormatStringFromTable( "GuildStrings", StringTables.Guild.DATE_FORMAT_ROSTER_LOGIN_X_DAYS_AGO, {tempRoundedNumber} ) 
	end	

	-- insert a 0 at the beginning for better formatting  (1/2/2008 becomes 01/02/2008
	if _day   < 10 then _day   = L"0".._day   end	
	if _month < 10 then _month = L"0".._month end

	return GetFormatStringFromTable( "GuildStrings", StringTables.Guild.DATE_FORMAT_ROSTER_LOGIN_DD_MM_YY, {_day, _month, _year} )
end

local function UpdateNotesTotal( currentNumNotes, note )
  if( note and note ~= L"" and note ~= "" ) then
    return currentNumNotes + 1
  end
    
  return currentNumNotes
end

local function UpdateLUASpecificMemberData(index, memberData)
	-- Career Icon
	GuildWindowTabRoster.memberListData[index].careerIcon = Icons.GetCareerIconIDFromCareerNamesID(memberData.careerID)

	-- Location Status
  if (not GuildWindowTabRoster.IsMemberOnline(memberData)) then -- For offline members, show their last login date instead.
    local dateStr = FormatLastLoginTime(memberData.lastLogin, memberData.lastLoginNumberHour24, memberData.lastLoginNumberMonth, memberData.lastLoginNumberDay, memberData.lastLoginNumberYear)
		GuildWindowTabRoster.memberListData[index].statusString = dateStr
  else	-- otherwise, show the zone name
		GuildWindowTabRoster.memberListData[index].statusString = GetZoneName (memberData.zoneID)
  end
end

local function InitMemberListData()
  GuildWindowTabRoster.memberListData = {}
  numOfficerNotes             = 0
  numGuildMemberNotes         = 0
  numRecruiters               = 0
    
	GuildWindowTabRoster.memberListData = GetGuildMemberData()
	
	for index, memberData in pairs( GuildWindowTabRoster.memberListData ) do
		UpdateLUASpecificMemberData(index, memberData)

		numGuildMemberNotes = UpdateNotesTotal( numGuildMemberNotes, memberData.note )
		numOfficerNotes = UpdateNotesTotal( numOfficerNotes, memberData.onote )
    if memberData.recruiterStatus > 0 then
      numRecruiters = numRecruiters + 1
    end
	end
end

-- Returns a table containing the Icon data {texture, x, and y} based on the abilityID. Returns a default empty icon upon any failures.
local function GetIconDataFromTacticID(tacticID)
  -- If we've purchased a tactic, the tacticID will be the ID of that tactic, otheriwse it will be <= 0
  if (tacticID ~= nil and tacticID > 0) then    
    local abilityData = GuildTacticsList.GetTacticData (tacticID)
    if (abilityData ~= nil) then
      local iconData = {}
      iconData.texture, iconData.x, iconData.y = GetIconData(abilityData.iconNum)
      if ((iconData.texture ~= nil) and (iconData.texture ~= "")) then
        return iconData
      end
    end
  end
  return GuildWindowTabBanner.GUILD_TACTIC_ICON_GREY
end


function GuildWindowFix.initialize()
  local hook0 = GuildWindowTabBanner.UpdatePurchasedTacticIcons
  GuildWindowTabBanner.UpdatePurchasedTacticIcons = GuildWindowFix.UpdatePurchasedTacticIcons
  
  local hook1 = GuildWindowFix.OnMouseOverTacticSlot
  GuildWindowTabBanner.OnMouseOverTacticSlot = GuildWindowFix.OnMouseOverTacticSlot
  
  local hook2 = GuildWindow.ToggleShowing
  GuildWindow.ToggleShowing = GuildWindowFix.ToggleShowing

end

function GuildWindowFix.ToggleShowing()
  InitMemberListData()
  GuildWindowTabRoster.UpdateMembersOnlineText()
  WindowUtils.ToggleShowing( "GuildWindow" )
end


function GuildWindowFix.OnMouseOverTacticSlot()
  local playerTitleNumber = GuildWindowTabAdmin.GetLocalMemberTitleNumber()

	-- If the player doesn't have permission to ever edit the Guild Standard, hide the save and cancel buttons
	local bCanEditStandard  = GuildWindowTabAdmin.GetGuildCommandPermission(SystemData.GuildPermissons.BANNER_MANAGEMENT, playerTitleNumber)
  local bIsStandardLocked = (GuildWindowTabBanner.Banners[GuildWindowTabBanner.CurrentBannerNumber].lastEdited > 0)
  local tacticSlotNumber = WindowGetId(SystemData.MouseOverWindow.name)
  local tacticID = GuildWindowTabBanner.Banners[GuildWindowTabBanner.CurrentBannerNumber].AbilityID[tacticSlotNumber]
  local abilityData = GuildTacticsList.GetTacticData( tacticID )
  if (abilityData ~=nil) then
    if (bCanEditStandard == true) then
      if (bIsStandardLocked == false) then
        Tooltips.CreateAbilityTooltip( abilityData, SystemData.ActiveWindow.name, GuildTacticsList.ABILITY_POPUP_ANCHOR, GetGuildString(StringTables.Guild.TOOLTIP_BANNER_CLEAR_TACTIC) )
      else
        Tooltips.CreateAbilityTooltip( abilityData, SystemData.ActiveWindow.name, GuildTacticsList.ABILITY_POPUP_ANCHOR, GetGuildString(StringTables.Guild.TOOLTIP_BANNER_LOCKED) )
      end
    else
      Tooltips.CreateAbilityTooltip( abilityData, SystemData.ActiveWindow.name, GuildTacticsList.ABILITY_POPUP_ANCHOR, GetGuildString(StringTables.Guild.TOOLTIP_BANNER_NO_PERMISSION) )
    end
  elseif (GuildWindowTabBanner.isMovingFromActiveSlot == false) then
    Tooltips.CreateTextOnlyTooltip (SystemData.MouseOverWindow.name, nil)
		-- If the guild has already unlocked this slot, tell the user how to assign a tactic to it.
		if GameData.Guild.m_GuildRank >= GuildWindowTabBanner.TacticUnlocks[GuildWindowTabBanner.CurrentBannerNumber][tacticSlotNumber] then
			Tooltips.SetTooltipText (1, 1, GetGuildString(StringTables.Guild.TOOLTIP_HOW_TO_ASSIGN_GUILD_TACTIC) )
		else	-- Otherwise, tell them when the slot unlocks.
			Tooltips.SetTooltipText (1, 1, GetFormatStringFromTable( "guildstrings", StringTables.Guild.TOOLTIP_BANNER_X_TACTIC_UNLOCKS_AT_RANK_Y, { tacticSlotNumber, GuildWindowTabBanner.TacticUnlocks[GuildWindowTabBanner.CurrentBannerNumber][tacticSlotNumber] } ) )
		end
    Tooltips.SetTooltipColorDef (1, 1, Tooltips.COLOR_HEADING)  
    Tooltips.Finalize ()
        
    local anchor = { Point="bottom", RelativeTo=SystemData.MouseOverWindow.name, RelativePoint="topleft", XOffset=0, YOffset=20 }
    Tooltips.AnchorTooltip (anchor)
    Tooltips.SetTooltipAlpha (1)
  end
end


function GuildWindowFix.UpdatePurchasedTacticIcons()
  local selectedBannerTacticSlotNumber = 0
  local tacticID = 0
  local iconData = nil
  local slotNumber = 0
  local tacticUsed = false
  local tacticUpgraded = false
  local previousTacticID = 0
  local upgradedTacticIndex = 0

	GuildWindowTabBanner.PurchasedTacticIconIDs = {}

  local rewardList = GuildWindowTabRewards.GetRewardList()
  local tacticPointRewardIndexMap = GuildWindowTabRewards.GetRewardTacticPointRewardIndexMap()
  local rewardData

	-- Loop through the only the tactic rewards
  for k, rewardIndex in pairs(tacticPointRewardIndexMap) do
    rewardData = rewardList[rewardIndex]
    tacticUsed = false
    tacticUpgraded = false
    previousTacticID = 0
    upgradedTacticIndex = 0
    tacticID = rewardData.tacticID
    --d(tacticID)
		-- If the tacticID is a positive number, then that means the tactic with the stored ID was purchased.
    if tacticID > 0 then
      iconData = GetIconDataFromTacticID(tacticID)

			-- Check if the tactic has prereqs becasue we only want to show the highest level purchased, not any of its prereqs.
      if (GuildTacticsList.HasPrereqBeenPurchased(tacticID)) then
        previousTacticID = GuildTacticsList.GetPrereqTacticID(tacticID)
        if (previousTacticID > 0) then
          for idx = 1, GuildWindowTabBanner.TOTAL_NUMBER_OF_AVAILABLE_TACTICS do
            -- Find this tactic id in the list
            if (GuildWindowTabBanner.PurchasedTacticIconIDs[idx] == previousTacticID and (idx < 19)) then
              tacticUpgraded = true
              GuildWindowTabBanner.PurchasedTacticIconIDs[idx] = tacticID
              DynamicImageSetTexture("GWRewardsBannerAvailTacticSlot"..idx.."Icon", iconData.texture, iconData.x, iconData.y )
              ButtonSetDisabledFlag("GWRewardsBannerAvailTacticSlot"..idx, false)
              WindowSetTintColor("GWRewardsBannerAvailTacticSlot"..idx.."Icon", 255, 255, 255)
              upgradedTacticIndex = idx
              break
            end
          end
        end
      end
            
      for idxStandard = 1, GuildWindowTabBanner.TOTAL_NUMBER_OF_STANDARDS do
        for slotNum = 1, GuildWindowTabBanner.Banners[idxStandard].numberUnlockedAbilitySlots do
          if (tacticUpgraded == true) then
            if (GuildWindowTabBanner.Banners[idxStandard].AbilityID[slotNum] == previousTacticID or GuildWindowTabBanner.Banners[idxStandard].AbilityID[slotNum] == tacticID) then
              GuildWindowTabBanner.Banners[idxStandard].AbilityID[slotNum] = tacticID
              for idx = 1, GuildWindowTabBanner.TOTAL_NUMBER_OF_AVAILABLE_TACTICS do
                if (GuildWindowTabBanner.PurchasedTacticIconIDs[idx] == tacticID and (idx < 19)) then
                  ButtonSetDisabledFlag("GWRewardsBannerAvailTacticSlot"..idx, true)
                  WindowSetTintColor("GWRewardsBannerAvailTacticSlot"..idx.."Icon", 92, 92, 92)
                end
              end
              break
            end
          else
            if (GuildWindowTabBanner.Banners[idxStandard].AbilityID[slotNum] == tacticID) then
              tacticUsed = true
              break  
            end
          end
        end
      end

      if (tacticUpgraded == false) then
        slotNumber = slotNumber +1
        if (slotNumber < 19) then
          DynamicImageSetTexture("GWRewardsBannerAvailTacticSlot"..slotNumber.."Icon", iconData.texture, iconData.x, iconData.y )
          if (tacticUsed == false) then
            ButtonSetDisabledFlag("GWRewardsBannerAvailTacticSlot"..slotNumber, false)
            WindowSetTintColor("GWRewardsBannerAvailTacticSlot"..slotNumber.."Icon", 255, 255, 255)
          else
            -- Make sure that all buttons that have been slotted are Disabled and Darkened
            ButtonSetDisabledFlag("GWRewardsBannerAvailTacticSlot"..slotNumber, true)
            WindowSetTintColor("GWRewardsBannerAvailTacticSlot"..slotNumber.."Icon", 92, 92, 92)
          end
                    
          GuildWindowTabBanner.PurchasedTacticIconIDs[slotNumber] = tacticID
        end
      end
    end
  end

    -- Fill in any empty slots remaining
  iconData = GuildWindowTabBanner.GUILD_TACTIC_ICON_GREY
  for slotNumber = slotNumber+1, GuildWindowTabBanner.TOTAL_NUMBER_OF_AVAILABLE_TACTICS do
    if (slotNumber < 19) then
      DynamicImageSetTexture("GWRewardsBannerAvailTacticSlot"..slotNumber.."Icon", iconData.texture, iconData.x, iconData.y )
      WindowSetTintColor("GWRewardsBannerAvailTacticSlot"..slotNumber.."Icon", 255, 255, 255)
      ButtonSetDisabledFlag("GWRewardsBannerAvailTacticSlot"..slotNumber, true)
    end
  end

end
